#include <iostream>
#include <fstream>
#include <cmath>



double expo(double x)
{
  
  if (x<0)
	return 1./expo(-x);
  /*
	Dans le cas x<0, il vaut mieux utiliser la relation exp(-x) = 1/exp(x). En effet, si x<0, le terme
	general de la serie entiere de l'exponentielle change alternativement de signe, ce qui peut etre
	tres mauvais pour les erreurs d'arrondi. Par exemple, les premiers termes de la serie pout exp(-100) seront
	1 - 100 + 5000 - 1.66x10^5 + 4.166x10^6 + ...
	pour une somme totale de 3x10^(-44).
	Autrement dit le nombre a calculer est totalement negligeable devant les erreurs d'arrondi.
  */

  
  double y = 0;
  double xn_n = 1;
  int n;
  
  
  // Dans la boucle suivante, xn_n vaudra a chaque fois x^n/n!
  for (n=1; y+xn_n != y; n++)
	/*
	  La condition y+xn_n != y utilise le fait que pour n assez grand, xn_n est tres petit par rapport a
	  y, de sorte que y+xn_n est exactement egal a y, a cause des erreurs d'arrondi.
	  Par consequent, numeriquement, la serie des x^n/n! est stationnaire a partir d'un certain rang.

	  On aurait pu aussi arreter la boucle quand n atteint une certaine valeur N.
	*/
	{
	  y += xn_n;
	  xn_n *= x/n;
	}
  return y;
}




int main(void)
{
  std::ofstream sortie("expo.dat");

  for (double x=-4 ; x<4 ; x+=0.01)
	sortie << x << "\t" << expo(x) << std::endl;
  sortie.close();



  // On calcule maintenant l'erreur relative entre exp(x) (la fonction fournie par la bibliotheque cmath) et la fonction expo(x) ci-dessus.
  
  double erreur = 0, x_erreur = 0;
  for (double x=-100; x<100; x+=0.25)
	{
	  // On cherche le point (de [-100,100]) pour lequel l'erreur relative est la plus elevee.
	  if (fabs(exp(x)-expo(x))/exp(x) > erreur)
		{
		  x_erreur = x;
		  erreur = fabs(exp(x)-expo(x))/exp(x);
		}
	}
  std::cout << "L'erreur relative maximale entre exp et expo sur [-100,100] est de " << erreur << " (atteinte en x = " << x_erreur << ")\n";  
  
  return 0;
}






