#include <fstream>
#include <cstdlib>
#include <cmath>
#include <random>

/*
  On utilise la bibliotheque random, presente seulement depuis C++ 11.
  Pour compiler ce programme avec g++, ajouter l'option -std=c++11.

  Sinon, remplacer l'utilisation de mt19937 par l'utilisation de rand.
*/ 

const int N = 1000; // Nombre de pas de temps
const double dt  = 1./N; // Pas de temps


double gauss(std::mt19937 &generateur)
{
  std::normal_distribution<double> X(0,1);
  // 0 et 1 correspondent à la moyenne et l'écart-type de la Gaussienne
  return X(generateur);
}


int main(void)
{
  std::mt19937 gen(2364); // On initialise un generateur avec une valeur quelconque.

  double t = 0, W = 0;
  // t va contenir le temps courant.
  // W va contenir la valeur du Brownien a l'instant t.

  std::ofstream sortie("brownien.dat");
  sortie << t << " " << W << "\n";

  for (int i=1; i<=N; i++)
	{
	  t = i*dt;
	  W += sqrt(dt) * gauss(gen);
	  /*
		L'accroissement d'un Brownien sur un intervalle de
		 largeur dt a une variance de dt. C'est donc dt^1/2
		 fois une Gaussienne centree reduite.
	  */
	  
	  sortie << t << " " << W << "\n";	  
	}
  sortie.close();
  return 0;
}
